VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "PresentValueOfAnnuity"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Option Explicit

Private m_objPayoutDetails As Payouts
Private m_curPayoutAmount As Currency
Private m_dblRate As Double
Private m_lngNumPayouts As Long
Private m_lngPayoutsPerYear As Long
Private m_curPresentValue As Currency
Private m_curTotalEarnings As Currency
Private m_curTotalPayouts As Currency

' setup constants for error handling
Private Const INVALIDAMOUNTERR As Long = vbObjectError + 1001
Private Const INVALIDAMOUNT As String = _
    "The amount of the loan must be specified"
Private Const INVALIDRATEERR As Long = vbObjectError + 1002
Private Const INVALIDRATE As String = _
    "The specified interest rate is invalid"
Private Const INVALIDNUMPAYOUTSERR As Long = vbObjectError + 1003
Private Const INVALIDNUMPAYOUTS As String = _
    "The number of payments must be greater than 0"
Private Const INVALIDPAYOUTSPERERR As Long = vbObjectError + 1004
Private Const INVALIDPAYOUTSPER As String = _
    "There must be at least one payment due each year"

' Create class event handlers

Private Sub Class_Initialize()
    ' create the Payments objects that will maintain the list
    ' of information about each payment
    Set m_objPayoutDetails = New Payouts
End Sub

Private Sub Class_Terminate()
    ' ensure that the memory for the objects is freed
    Set m_objPayoutDetails = Nothing
End Sub

' Create the properties for our class

' the PayoutAmount property indicates the amount to be paid each period
Public Property Get PayoutAmount() As Currency
    PayoutAmount = m_curPayoutAmount
End Property

Public Property Let PayoutAmount(curAmount As Currency)
    ' check for valid property value
    If curAmount <= 0 Then
        Err.Raise vbObjectError + 1001, _
    "FinancialUtility:Amortization:Amount", INVALIDAMOUNT
    End If
    m_curPayoutAmount = curAmount
End Property

' the Rate property is the interest rate for the annuity
' it is expressed in percentages - for example 6.5 is 6.5% or .065 per year
Public Property Get Rate() As Double
    Rate = m_dblRate
End Property

Public Property Let Rate(dblRate As Double)
    ' check for valid property value
    If dblRate <= 0 Or dblRate > 50 Then
        Err.Raise INVALIDRATEERR, _
    "FinancialUtility:Amortization:Calculate", INVALIDRATE
    End If
    m_dblRate = dblRate
End Property

' the NumberOfPayouts property identifies the number of
' payouts that are scheduled to be made over the life of
' the annuity
Public Property Get NumberOfPayouts() As Long
    NumberOfPayouts = m_lngNumPayouts
End Property

Public Property Let NumberOfPayouts(lngPayouts As Long)
    ' check for valid property value
    If lngPayouts <= 0 Then
        Err.Raise INVALIDNUMPAYOUTSERR, _
    "FinancialUtility:Amortization:Calculate", INVALIDNUMPAYOUTS
    End If
    m_lngNumPayouts = lngPayouts
End Property

' the PayoutsPerYear property identifies the number of payouts
' that are required each year - used to calculate the periodic rate
Public Property Get PayoutsPerYear() As Long
    PayoutsPerYear = m_lngPayoutsPerYear
End Property

Public Property Let PayoutsPerYear(lngPayoutsPerYear As Long)
    ' check for valid property value
    If lngPayoutsPerYear <= 0 Then
        Err.Raise INVALIDPAYOUTSPERERR, _
    "FinancialUtility:Amortization:Calculate", INVALIDPAYOUTSPER
    End If
    m_lngPayoutsPerYear = lngPayoutsPerYear
End Property

' the PresentValue property is set when the calculate method
' is called - this is a read only property that tells the
' required initial investment for the annuity
Public Property Get PresentValue() As Currency
    PresentValue = m_curPresentValue
End Property

' the TotalEarnings property is set when the calculate method
' is called - this is a read only property that tells the
' total amount of earnings over the life of the annuity
Public Property Get TotalEarnings() As Currency
    TotalEarnings = m_curTotalEarnings
End Property

' the TotalPayouts property is set when the calculate method
' is called - this is a read only property that tells the
' total amount of payouts over the life of the annuity
Public Property Get TotalPayouts() As Currency
    TotalPayouts = m_curTotalPayouts
End Property

' the PayouttDetails property returns a reference to a Payouts
' object that holds information about each scheduled payout
' for the annuity
Public Property Get PayoutDetails() As Payouts
    Set PayoutDetails = m_objPayoutDetails
End Property

' Create the methods for our class

' the calculate method is the only method for our class
' but it is the work horse where all of the information is
' generated
' it accepts optional parameters and sets the properties
' accordingly if they are passed in
Public Sub Calculate(Optional curPayoutAmount As Currency, _
Optional dblRate As Double, Optional lngNumPayouts As Long, _
Optional lngPayoutsPerYear As Long)

    ' declare local variables
    Dim dblPeriodicRate As Double
    Dim dblFactor2 As Double
    Dim dblFactor4 As Double
    Dim curCurrentBalance As Currency
    Dim lngCounter As Long
    Dim objPayout As Payout

    On Error GoTo CALCULATE_ERROR

    ' check to see if we need to set the properties to the input values
    If curPayoutAmount <> 0 Then m_curPayoutAmount = curPayoutAmount
    If dblRate <> 0 Then m_dblRate = dblRate
    If lngNumPayouts <> 0 Then m_lngNumPayouts = lngNumPayouts
    If lngPayoutsPerYear <> 0 Then m_lngPayoutsPerYear = _
        lngPayoutsPerYear

    ' ensure that we have the required information or send an
    ' error to the user
    If m_curPayoutAmount <= 0 Then
        Err.Raise INVALIDAMOUNTERR, _
    "FinancialUtility:Amortization:Calculate", INVALIDAMOUNT
    End If
    
    If m_dblRate <= 0 Or dblRate > 50 Then
        Err.Raise INVALIDRATEERR, _
    "FinancialUtility:Amortization:Calculate", INVALIDRATE
    End If
    
    If m_lngNumPayouts <= 0 Then
        Err.Raise INVALIDNUMPAYOUTSERR, _
    "FinancialUtility:Amortization:Calculate", INVALIDNUMPAYOUTS
    End If
    
    If m_lngPayoutsPerYear <= 0 Then
        Err.Raise INVALIDPAYOUTSPERERR, _
    "FinancialUtility:Amortization:Calculate", INVALIDPAYOUTSPER
    End If
    
    ' calculate the periodic interest rate based on the rate
    ' and number of payouts per year
    dblPeriodicRate = (m_dblRate / 100 / m_lngPayoutsPerYear)

    'Calculate Periodic Payout
    ' use the periodic rate and other inputs to calculate
    ' the required payment.  This is broken into steps
    ' to facilitate better understanding
    dblFactor2 = 1 / (1 + dblPeriodicRate) ^ m_lngNumPayouts
    dblFactor4 = (1 - dblFactor2) / dblPeriodicRate
    m_curPresentValue = m_curPayoutAmount * dblFactor4

    ' compute each payout for the annuity
    ' keep track of the current balance so we can
    ' identify each payouts details
    curCurrentBalance = m_curPresentValue

    'Calculate TotalEarnings and Details for each payout
    For lngCounter = 1 To m_lngNumPayouts
        ' for each payout create a Payout object
        ' and set its properties
        Set objPayout = New Payout

        With objPayout
            .PayoutNumber = lngCounter
            .EarningsForPeriod = curCurrentBalance * dblPeriodicRate
            ' keep a running total of overall Earnings
            m_curTotalEarnings = m_curTotalEarnings + .EarningsForPeriod
            .ReturnOfCapital = m_curPayoutAmount - .EarningsForPeriod
            .RemainingBalance = curCurrentBalance - .ReturnOfCapital

            curCurrentBalance = .RemainingBalance
        End With

        ' add the new payout to the Payouts collection
        m_objPayoutDetails.Add objPayout
        ' free our object
        ' VB does not always work correctly if we don't
        Set objPayout = Nothing
    Next

    ' set the TotalPayouts property
    m_curTotalPayouts = m_curPayoutAmount * m_lngNumPayouts

    Exit Sub

CALCULATE_ERROR:
    ' if required add specific error handling here
    ' for now we will just raise all errors to the calling
    ' program
    Err.Raise Err.Number

End Sub

'Create supporting procedures for our class
'N/A


